#include "dEvian.h"

static E_Config_DD *_edd_main;
static E_Config_DD *_edd_devians;
static E_Config_DD *_edd_rss_docs;

static E_Config_DD *_devian_config_main_edd_init();
static E_Config_DD *_devian_config_devians_edd_init();
static E_Config_DD *_devian_config_rss_docs_edd_init();

/* Public functions */

Config_dEvian_Main *devian_config_init(void)
{
  Config_dEvian_Main *conf;

  _edd_devians = _devian_config_devians_edd_init();
  if (!_edd_devians)
    return NULL;
  _edd_rss_docs = _devian_config_rss_docs_edd_init();
  if (!_edd_rss_docs)
    return NULL;
  _edd_main = _devian_config_main_edd_init();
  if (!_edd_main)
    return NULL;

  conf = devian_config_load();
  if (!conf)
    return NULL;

  return conf;
}

Config_dEvian_Main *devian_config_load(void)
{
  Config_dEvian_Main *cfg_main;
  
  cfg_main = e_config_domain_load("module.devian", _edd_main);

  /* If loaded, check for good version */
  if (cfg_main)
    {
      if (cfg_main->conf_version < CONFIG_VERSION)
	{  
	  char buf[4096];
	  snprintf(buf, sizeof(buf),
		   _(
		     "<hilight>Configuration Upgraded</hilight><br><br>"
		     "Your configuration of dEvian module<br>"
		     "has been upgraded<br>"
		     "Your settings were removed<br>"
		     "Sorry for the inconvenience<br><br>"
		     "(%d -> %d)"),
		   cfg_main->conf_version,
		   CONFIG_VERSION);
	  e_module_dialog_show(_("dEvian Module"), buf);
	  
	  cfg_main = NULL;
	}
      else
	{
	  if (cfg_main->conf_version > CONFIG_VERSION)
	    {
	      char buf[4096];
	      snprintf(buf, sizeof(buf),
		       _(
			 "<hilight>Configuration Downgraded</hilight><br><br>"
			 "Your configuration of dEvian module<br>"
			 "has been downgraded<br>"
			 "Your settings were removed<br>"
			 "Sorry for the inconvenience<br><br>"
			 "(%d ->%d)"),
		       cfg_main->conf_version,
		       CONFIG_VERSION);
	      e_module_dialog_show(_("dEvian Module"), buf);
	      cfg_main = NULL;
	    }
	}
    }

  if (!cfg_main)
    cfg_main = devian_config_main_new();

  if (cfg_main->nb_devian != evas_list_count(cfg_main->devians_conf))
    {
      fprintf(stderr, "dEvian: nb_devian (%d) != count(devians_conf) (%d) !!!\n",
	      cfg_main->nb_devian, evas_list_count(cfg_main->devians_conf));
      cfg_main->nb_devian = evas_list_count(cfg_main->devians_conf);
    }

  E_CONFIG_LIMIT(cfg_main->high_quality, 0, 1);
  //... ADD all config limits

  DMAIN(("dEvian: config initialised"));

  return cfg_main;
}

void devian_config_save(void)
{
  Config_dEvian_Main *cfg_main;
  Evas_List *l;
  dEvian *devian;

  cfg_main = dEvianM->conf;
  if (!cfg_main)
    return;

  /* Save number of devians */
  cfg_main->nb_devian = evas_list_count(dEvianM->devians);

  /* Save config of each devian in main config */
  cfg_main->devians_conf = NULL;
  for (l = dEvianM->devians; l; l=evas_list_next(l))
    {
      devian = evas_list_data(l);
      devian_config_devian_save(devian);
      cfg_main->devians_conf = evas_list_append(cfg_main->devians_conf,
						devian->conf);
      DMAIN(("dEvian Config saved"));
    }
  
  e_config_domain_save("module.devian", _edd_main, dEvianM->conf);
}

void devian_config_devian_save(dEvian *devian)
{
  Container_Box *box;

  if (!devian->conf)
    return;

  if (devian->conf->container_type == CONTAINER_BOX)
    {
      box = devian->container;
      devian->conf->box_x = box->x + box->theme_border_w/2;
      devian->conf->box_y = box->y + box->theme_border_h/2;
      devian->conf->box_w = box->go_w - box->theme_border_w;
      devian->conf->box_h = box->go_h - box->theme_border_h;
    }
}


void devian_config_free(void)
{
  Evas_List *l;

  Config_dEvian_Main *cfg_main;

  cfg_main = dEvianM->conf;

  evas_stringshare_del(cfg_main->viewer_http);
  evas_stringshare_del(cfg_main->viewer_image);
  evas_stringshare_del(cfg_main->theme_picture);
  evas_stringshare_del(cfg_main->theme_rss);
  evas_stringshare_del(cfg_main->theme_popup);
  evas_stringshare_del(cfg_main->sources_picture_data_import_dir);

  for (l = cfg_main->sources_rss_docs; l; l=evas_list_next(l))
    devian_data_rss_doc_free(evas_list_data(l), 0, 1);

  for (l = dEvianM->devians; l; l=evas_list_next(l))
    devian_config_devian_free(evas_list_data(l));

  E_FREE(dEvianM->conf);

  E_CONFIG_DD_FREE(_edd_main);
  E_CONFIG_DD_FREE(_edd_devians);
  E_CONFIG_DD_FREE(_edd_rss_docs);
}

void devian_config_devian_free(dEvian *devian)
{
  if (!devian->conf)
    return;

  if (devian->conf->rss_url)
  evas_stringshare_del(devian->conf->rss_url);

  E_FREE(devian->conf);
  devian->conf = NULL;
}

Config_dEvian_Main *devian_config_main_new(void)
{
  Config_dEvian_Main *cfg_main;
  char buf[DEVIAN_MAX_PATH];

  /* Create new config */
  DMAIN(("dEvian: New config creation ..."));
  cfg_main = E_NEW(Config_dEvian_Main, 1);
  cfg_main->conf_version = CONFIG_VERSION;
  cfg_main->high_quality = 0;
  cfg_main->nb_devian = 0;
  cfg_main->theme_picture = evas_stringshare_add(DEVIAN_THEME_DEFAULT);
  cfg_main->theme_rss = evas_stringshare_add(DEVIAN_THEME_DEFAULT);
  cfg_main->theme_popup = evas_stringshare_add(DEVIAN_THEME_DEFAULT);
  
  snprintf(buf, DEVIAN_MAX_PATH, "%s/Tofs", e_user_homedir_get());
  cfg_main->sources_picture_data_import_dir = (char *)evas_stringshare_add(buf);
  cfg_main->sources_picture_data_import_thumbs_warn = 1;
  cfg_main->sources_picture_data_import_recursive = 0;
  cfg_main->sources_picture_default_location = DATA_PICTURE_LOCAL;
  cfg_main->sources_picture_set_bg_purge = SOURCE_PICTURE_SET_BG_PURGE_DEFAULT;
  cfg_main->sources_rss_timer_s = SOURCE_RSS_UPDATE_RATE_DEFAULT;
  cfg_main->sources_rss_popup_news = POPUP_WARN_DEVIAN_ACTIVE_DEFAULT;
  cfg_main->sources_rss_popup_news_timer = POPUP_WARN_TIMER_DEFAULT;

  {
    Rss_Doc *doc;

    doc = E_NEW(Rss_Doc, 1);
    doc->name = evas_stringshare_add("E cvs");
    doc->url = evas_stringshare_add("http://cia.navi.cx/stats/project/e/.rss?ver=2&amp;medium=plaintext&amp;limit=");
    doc->host = evas_stringshare_add("cia.navi.cx");
    doc->description = evas_stringshare_add("Enlightenment cvs");
    doc->version = DATA_RSS_DOC_VERSION_DEFAULT;
    doc->w_name = DATA_RSS_DOC_W_NAME_DEFAULT;
    doc->w_description = DATA_RSS_DOC_W_DESCRIPTION_DEFAULT;
    doc->user = NULL;
    doc->state = DATA_RSS_DOC_STATE_USABLE;   
    cfg_main->sources_rss_docs =
      evas_list_append(cfg_main->sources_rss_docs, doc);

    doc = E_NEW(Rss_Doc, 1);
    doc->name = evas_stringshare_add("Gentoo packages amd64");
    doc->url = evas_stringshare_add("http://packages.gentoo.org/archs/amd64/gentoo.rss");
    doc->host = evas_stringshare_add("packages.gentoo.org");
    doc->description = evas_stringshare_add("");
    doc->version = DATA_RSS_DOC_VERSION_DEFAULT;
    doc->w_name = DATA_RSS_DOC_W_NAME_DEFAULT;
    doc->w_description = DATA_RSS_DOC_W_DESCRIPTION_DEFAULT;
    doc->user = NULL;
    doc->state = DATA_RSS_DOC_STATE_USABLE;   
    cfg_main->sources_rss_docs =
      evas_list_append(cfg_main->sources_rss_docs, doc);

    doc = E_NEW(Rss_Doc, 1);
    doc->name = evas_stringshare_add("OSNews");
    doc->url = evas_stringshare_add("http://www.osnews.com/files/recent.rdf");
    doc->host = evas_stringshare_add("www.osnews.com");
    doc->description = evas_stringshare_add("");
    doc->version = DATA_RSS_DOC_VERSION_DEFAULT;
    doc->w_name = DATA_RSS_DOC_W_NAME_DEFAULT;
    doc->w_description = DATA_RSS_DOC_W_DESCRIPTION_DEFAULT;
    doc->user = NULL;
    doc->state = DATA_RSS_DOC_STATE_USABLE;   
    cfg_main->sources_rss_docs =
      evas_list_append(cfg_main->sources_rss_docs, doc);

    doc = E_NEW(Rss_Doc, 1);
    doc->name = evas_stringshare_add("Yahoo! News");
    doc->url = evas_stringshare_add("http://rss.news.yahoo.com/rss/topstories");
    doc->host = evas_stringshare_add("rss.news.yahoo.com");
    doc->description = evas_stringshare_add("");
    doc->version = DATA_RSS_DOC_VERSION_DEFAULT;
    doc->w_name = DATA_RSS_DOC_W_NAME_DEFAULT;
    doc->w_description = DATA_RSS_DOC_W_DESCRIPTION_DEFAULT;
    doc->user = NULL;
    doc->state = DATA_RSS_DOC_STATE_USABLE;   
    cfg_main->sources_rss_docs =
      evas_list_append(cfg_main->sources_rss_docs, doc);

    doc = E_NEW(Rss_Doc, 1);
    doc->name = evas_stringshare_add("Linux Today");
    doc->url = evas_stringshare_add("http://www.linuxtoday.com/backend/biglt.rss");
    doc->host = evas_stringshare_add("www.linuxtoday.com");
    doc->description = evas_stringshare_add("");
    doc->version = DATA_RSS_DOC_VERSION_DEFAULT;
    doc->w_name = DATA_RSS_DOC_W_NAME_DEFAULT;
    doc->w_description = DATA_RSS_DOC_W_DESCRIPTION_DEFAULT;
    doc->user = NULL;
    doc->state = DATA_RSS_DOC_STATE_USABLE;   
    cfg_main->sources_rss_docs =
      evas_list_append(cfg_main->sources_rss_docs, doc);

    doc = E_NEW(Rss_Doc, 1);
    doc->name = evas_stringshare_add("Freshmeat");
    doc->url = evas_stringshare_add("http://rss.freshmeat.net/freshmeat/feeds/fm-releases-global");
    doc->host = evas_stringshare_add("rss.freshmeat.net");
    doc->description = evas_stringshare_add("");
    doc->version = DATA_RSS_DOC_VERSION_DEFAULT;
    doc->w_name = DATA_RSS_DOC_W_NAME_DEFAULT;
    doc->w_description = DATA_RSS_DOC_W_DESCRIPTION_DEFAULT;
    doc->user = NULL;
    doc->state = DATA_RSS_DOC_STATE_USABLE;   
    cfg_main->sources_rss_docs =
      evas_list_append(cfg_main->sources_rss_docs, doc);

    doc = E_NEW(Rss_Doc, 1);
    doc->name = evas_stringshare_add("Slashdot");
    doc->url = evas_stringshare_add("http://rss.slashdot.org/Slashdot/slashdot");
    doc->host = evas_stringshare_add("rss.slashdot.org");
    doc->description = evas_stringshare_add("News for nerds, stuff that matters");
    doc->version = DATA_RSS_DOC_VERSION_DEFAULT;
    doc->w_name = DATA_RSS_DOC_W_NAME_DEFAULT;
    doc->w_description = DATA_RSS_DOC_W_DESCRIPTION_DEFAULT;
    doc->user = NULL;
    doc->state = DATA_RSS_DOC_STATE_USABLE;   
    cfg_main->sources_rss_docs =
      evas_list_append(cfg_main->sources_rss_docs, doc);

    doc = E_NEW(Rss_Doc, 1);
    doc->name = evas_stringshare_add("CNN Top stories");
    doc->url = evas_stringshare_add("http://rss.cnn.com/rss/cnn_topstories.rss");
    doc->host = evas_stringshare_add("rss.cnn.com");
    doc->description = evas_stringshare_add("");
    doc->version = DATA_RSS_DOC_VERSION_DEFAULT;
    doc->w_name = DATA_RSS_DOC_W_NAME_DEFAULT;
    doc->w_description = DATA_RSS_DOC_W_DESCRIPTION_DEFAULT;
    doc->user = NULL;
    doc->state = DATA_RSS_DOC_STATE_USABLE;
    cfg_main->sources_rss_docs =
      evas_list_append(cfg_main->sources_rss_docs, doc);
  }
  
  cfg_main->container_default = CONTAINER_BOX;
  cfg_main->boxs_always_id = CONTAINER_BOX_ALWAYS_ID_DEFAULT;
  cfg_main->boxs_always_name = CONTAINER_BOX_ALWAYS_NAME_DEFAULT;
  cfg_main->boxs_stacking = CONTAINER_BOX_STACKING_DEFAULT;
  cfg_main->boxs_default_max_size = CONTAINER_BOX_SIZE_DEFAULT;
  cfg_main->boxs_default_anim = CONTAINER_BOX_ANIM_DEFAULT;
  cfg_main->boxs_anim_ghost_timer = CONTAINER_BOX_ANIM_GHOST_TIMER_DEFAULT;
  cfg_main->boxs_nice_resize = CONTAINER_BOX_NICE_RESIZE_DEFAULT;
  cfg_main->boxs_nice_trans = CONTAINER_BOX_NICE_TRANS_DEFAULT;

  cfg_main->data_picture_thumb_default_size = DATA_PICTURE_THUMB_SIZE_DEFAULT;
  cfg_main->data_picture_cache_size = DATA_PICTURE_CACHE_SIZE_DEFAULT;

  cfg_main->viewer_http = evas_stringshare_add(DEVIAN_VIEWER_HTTP_DEFAULT);
  cfg_main->viewer_image = evas_stringshare_add(DEVIAN_VIEWER_IMAGE_DEFAULT);
  
  cfg_main->devians_conf = NULL;

  return cfg_main;
}

/*
  Creates a new config for a devian
  Args:
    - Conf: if !NULL, the returned config will be a copy of this one
  Return:
    The new config
*/
Config_dEvian *devian_config_devian_new(int source_type, Config_dEvian *conf)
{
  Config_dEvian *cfg;

  cfg = E_NEW(Config_dEvian, 1);

  if (conf)
    {
      cfg->source_type = source_type;
      cfg->picture_timer_active = conf->picture_timer_active;
      cfg->picture_timer_s = conf->picture_timer_s;
      if (conf->rss_url)
	cfg->rss_url = evas_stringshare_add(conf->rss_url);
      else
	cfg->rss_url = NULL;
      cfg->rss_doc = NULL;
      cfg->rss_timer_s = conf->rss_timer_s;
      cfg->rss_timer_active = conf->rss_timer_active;
      cfg->rss_nb_items = conf->rss_nb_items;
      cfg->rss_popup_news = conf->rss_popup_news;

      cfg->container_type = conf->container_type;
      cfg->box_speed = conf->box_speed;
      cfg->box_max_size_source = conf->box_max_size_source;
      cfg->box_x = conf->box_x;
      cfg->box_y = conf->box_y;
      cfg->box_w = conf->box_w;
      cfg->box_h = conf->box_h;
      cfg->box_auto_resize = conf->box_auto_resize;
      cfg->box_anim = conf->box_anim;
      cfg->box_allow_overlap = conf->box_allow_overlap;
      cfg->box_infos_show = conf->box_infos_show;
      cfg->box_infos_pos = conf->box_infos_pos;
    }
  else
    {
      cfg->source_type = source_type;
      cfg->picture_timer_active = 1;
      cfg->picture_timer_s = SOURCE_PICTURE_TIMER_DEFAULT;
      cfg->rss_doc = devian_data_rss_doc_find_unused();
      if (cfg->rss_doc)
	cfg->rss_url = evas_stringshare_add(cfg->rss_doc->url);
      else
	cfg->rss_url = NULL;
      cfg->rss_timer_s = dEvianM->conf->sources_rss_timer_s;
      cfg->rss_timer_active = 1;
      cfg->rss_nb_items = SOURCE_RSS_NB_ITEMS_DEFAULT;
      cfg->rss_popup_news = POPUP_WARN_DEVIAN_ACTIVE_DEFAULT;
      
      cfg->container_type = dEvianM->conf->container_default;
      cfg->box_speed = CONTAINER_BOX_SPEED_DEFAULT;
      cfg->box_max_size_source = dEvianM->conf->boxs_default_max_size;
      devian_container_box_random_pos_get(NULL, &cfg->box_x, &cfg->box_y,
					  cfg->box_max_size_source);
      cfg->box_w = dEvianM->conf->boxs_default_max_size;
      cfg->box_h = dEvianM->conf->boxs_default_max_size;
      cfg->box_auto_resize = CONTAINER_BOX_AUTO_RESIZE_DEFAULT;
      cfg->box_anim = dEvianM->conf->boxs_default_anim;
      cfg->box_allow_overlap = CONTAINER_BOX_ALLOW_OVERLAP_DEFAULT;
      cfg->box_infos_show = CONTAINER_BOX_INFOS_SHOW_DEFAULT;
      cfg->box_infos_pos = CONTAINER_BOX_INFOS_POS_DEFAULT;
    }

  DMAIN(("New config created"));

  return cfg;
}

/* Private functions */

static E_Config_DD *_devian_config_main_edd_init(void)
{
  E_Config_DD *edd_main;

  edd_main = E_CONFIG_DD_NEW("dEvian_Config", Config_dEvian_Main);

#undef T
#undef D
#define T Config_dEvian_Main
#define D edd_main
  
  E_CONFIG_VAL(D, T, conf_version, INT);
  E_CONFIG_VAL(D, T, high_quality, INT);
  E_CONFIG_VAL(D, T, nb_devian, INT);
  E_CONFIG_VAL(D, T, theme_picture, STR);
  E_CONFIG_VAL(D, T, theme_rss, STR);
  E_CONFIG_VAL(D, T, theme_popup, STR);


  E_CONFIG_VAL(D, T, sources_picture_data_import_dir, STR);
  E_CONFIG_VAL(D, T, sources_picture_data_import_thumbs_warn, INT);
  E_CONFIG_VAL(D, T, sources_picture_data_import_recursive, INT);
  E_CONFIG_VAL(D, T, sources_picture_default_location, INT);
  E_CONFIG_VAL(D, T, sources_picture_set_bg_purge, INT);
  E_CONFIG_VAL(D, T, sources_rss_timer_s, INT);
  E_CONFIG_LIST(D, T, sources_rss_docs, _edd_rss_docs);
  E_CONFIG_VAL(D, T, sources_rss_popup_news, INT);
  E_CONFIG_VAL(D, T, sources_rss_popup_news_timer, INT);

  E_CONFIG_VAL(D, T, container_default, INT);

  E_CONFIG_VAL(D, T, boxs_always_id, INT);
  E_CONFIG_VAL(D, T, boxs_always_name, INT);
  E_CONFIG_VAL(D, T, boxs_stacking, INT);
  E_CONFIG_VAL(D, T, boxs_default_max_size, INT);
  E_CONFIG_VAL(D, T, boxs_default_anim, INT);
  E_CONFIG_VAL(D, T, boxs_anim_ghost_timer, INT);
  E_CONFIG_VAL(D, T, boxs_nice_resize, INT);
  E_CONFIG_VAL(D, T, boxs_nice_trans, INT);

  E_CONFIG_VAL(D, T, data_picture_thumb_default_size, INT);
  E_CONFIG_VAL(D, T, data_picture_cache_size, INT);

  E_CONFIG_VAL(D, T, viewer_http, STR);
  E_CONFIG_VAL(D, T, viewer_image, STR);

  E_CONFIG_LIST(D, T, devians_conf, _edd_devians);
   
  return edd_main;
}

static E_Config_DD *_devian_config_devians_edd_init(void)
{
  E_Config_DD *devians;

  devians = E_CONFIG_DD_NEW("dEvian_Config_dEvians", Config_dEvian);

#undef T
#undef D
#define T Config_dEvian
#define D devians

  E_CONFIG_VAL(D, T, source_type, INT);
  E_CONFIG_VAL(D, T, picture_timer_active, INT);
  E_CONFIG_VAL(D, T, picture_timer_s, INT);
  E_CONFIG_VAL(D, T, rss_url, STR);
  E_CONFIG_VAL(D, T, rss_timer_s, INT);
  E_CONFIG_VAL(D, T, rss_timer_active, INT);
  E_CONFIG_VAL(D, T, rss_nb_items, INT);
  E_CONFIG_VAL(D, T, rss_popup_news, INT);

  E_CONFIG_VAL(D, T, container_type, INT);

  E_CONFIG_VAL(D, T, box_speed, INT);
  E_CONFIG_VAL(D, T, box_x, INT);
  E_CONFIG_VAL(D, T, box_y, INT);
  E_CONFIG_VAL(D, T, box_w, INT);
  E_CONFIG_VAL(D, T, box_h, INT);
  E_CONFIG_VAL(D, T, box_max_size_source, INT);
  E_CONFIG_VAL(D, T, box_auto_resize, INT);
  E_CONFIG_VAL(D, T, box_anim, INT);
  E_CONFIG_VAL(D, T, box_allow_overlap, INT);
  E_CONFIG_VAL(D, T, box_infos_show, INT);
  E_CONFIG_VAL(D, T, box_infos_pos, INT);

  return devians;
}

static E_Config_DD *_devian_config_rss_docs_edd_init(void)
{
  E_Config_DD *rss_docs;

  rss_docs = E_CONFIG_DD_NEW("dEvian_Config_Rss_Docs", Rss_Doc);

#undef T
#undef D
#define T Rss_Doc
#define D rss_docs

  E_CONFIG_VAL(D, T, name, STR);
  E_CONFIG_VAL(D, T, url, STR);
  E_CONFIG_VAL(D, T, host, STR);
  E_CONFIG_VAL(D, T, description, STR);
  E_CONFIG_VAL(D, T, version, FLOAT);

  E_CONFIG_VAL(D, T, w_name, INT);
  E_CONFIG_VAL(D, T, w_description, INT);

  return rss_docs;
}
