#include "dEvian.h"

/**
 * Initalise dEvian
 * @param m Pointer to the module
 * @return 1 on success, 0 if fails
 */
int devian_devian_main_init(E_Module *m)
{
  Config_dEvian *cfg_devian;
  int i;

  dEvianM = E_NEW(dEvian_Main, 1);

  dEvianM->container = e_container_current_get(e_manager_current_get());
  evas_output_viewport_get(dEvianM->container->bg_evas, NULL, NULL, &dEvianM->canvas_w, &dEvianM->canvas_h);
  DMAIN(("dEvian : Canvas %d x %d",dEvianM->canvas_w,dEvianM->canvas_h));

  dEvianM->module = m;

  dEvianM->devians = NULL;
  dEvianM->container_box_count = 0;
  dEvianM->container_bar_count = 0;
  dEvianM->source_picture_count = 0;
  dEvianM->picture_list_local = NULL;
  dEvianM->picture_list_net = NULL;
  dEvianM->picture_cache = NULL;
  dEvianM->dialog_conf = NULL;
  dEvianM->dialog_conf_theme = NULL;

  /* Libs used */
  dEvianM->display = devian_display_init();

  /* Config */
  dEvianM->conf = devian_config_init();
  if (!dEvianM->conf)
      return 0;

  /* Init devians like it is in the config file */
  DMAIN(("%d devians to create", dEvianM->conf->nb_devian));
  for (i=0; i<dEvianM->conf->nb_devian; i++)
    {
      cfg_devian = evas_list_nth(dEvianM->conf->devians_conf, i);
      if (!devian_devian_add(cfg_devian->source_type, cfg_devian))
	{
	  /* Should never happend */
	  fprintf(stderr, "dEvian: Not good, a devian couldnt be created\n");
	  evas_list_remove(dEvianM->conf->devians_conf,
			   cfg_devian);
	}
    }

  DMAIN(("dEvian : init_devian OK\n"));

  return 1;
}

/**
 * Shutdown dEvian
 */
void devian_devian_main_shutdown(void)
{
  DMAIN(("Main shutdown begins"));

  /* dEvians */
  devian_devian_del_all();
  /* Popups */
  devian_popup_warn_del_all();
  /* Dialogs */
  devian_config_dialog_main_shutdown();
  /* Config */
  devian_config_free();
  /* Libs */
  devian_display_shutdown();


  E_FREE(dEvianM);
}

/**
 * Creates a new dEvian and add it to the devians list
 * @param source_type Type of the devian to create
 * @param cfg_devian Pointer to the config to use for new devian. If its NULL, we create a new config
 * @return: Pointer to the new dEvian or NULL if fails
 */
dEvian *devian_devian_add(int source_type, Config_dEvian *cfg_devian)
{
  dEvian *devian;

  devian = E_NEW(dEvian, 1);

  devian->id = NULL;

  devian->source = NULL;
  devian->source_func.timer_change = devian_source_idle_timer_change;
  devian->source_func.refresh = devian_source_idle_refresh;
  devian->source_func.set_bg = devian_source_idle_set_bg;
  devian->source_func.viewer = devian_source_idle_viewer;
  devian->source_func.gui_update = devian_source_idle_gui_update;

  devian->size_policy = SIZE_POLICY_DEFAULT;
  devian->popup_warn = NULL;

  devian->container = NULL;
  devian->container_func.resize_auto = devian_container_idle_resize_auto;
  devian->container_func.update_actions = devian_container_idle_update_actions;
  devian->container_func.is_in_transition = devian_container_idle_is_in_transition;

  devian->dying = 0;
  devian->conf = NULL;
  devian->dialog_conf = NULL;
  devian->dialog_conf_rss = NULL;
  devian->menu = NULL;

  /* Append new devian to the list of devians */
  dEvianM->devians = evas_list_append (dEvianM->devians,
				       devian);

  /* Id */
  devian_devian_set_id(devian, source_type, NULL);

  /* Config */
  if (cfg_devian)
    {
      /* Use existing config */
      devian->conf = cfg_devian;
    }
  else
    {
      /* New config */
      devian->conf = devian_config_devian_new(source_type, NULL);
      if (!devian->conf)
	{
	  devian_devian_del(devian, 1);
	  return NULL;
	}
    }
  
  /* Menu */
  devian->menu = devian_menu_init(devian);
  if (!devian->menu)
    {
      devian_devian_del(devian, 1);
      return NULL;
    }

  /* Source */
  if ( devian_source_change(devian, devian->conf->source_type) )
    {
      devian_devian_del(devian, 1);
      return NULL;
    }

  /* Container */
  if ( devian_container_change(devian, devian->conf->container_type) )
    {
      devian_devian_del(devian, 1);
      return NULL;
    }

  DMAIN(("dEvian: New devian OK"));

  e_config_save_queue();

  return devian;
}

/**
 * Delete a dEvian
 *
 * @param devian The dEvian to delete
 * @param now If 1, delete the dEvian without transition.
 */
void devian_devian_del(dEvian *devian, int now)
{
  if (!now)
    {
      devian->dying = 1;
      if (devian_container_devian_dying(devian))
	return;
    }
	
  DMAIN(("dEvian: del devian container"));
  devian_container_change(devian, CONTAINER_NO);
  DMAIN(("dEvian: del devian source"));
  devian_source_change(devian, SOURCE_NO);
  DMAIN(("dEvian: Free devian menu"));
  devian_menu_shutdown(devian);
  DMAIN(("dEvian: Free devian config dialog"));
  devian_config_dialog_devian_shutdown(devian);
  DMAIN(("dEvian: Free devian config"));
  devian_config_devian_free(devian);
  DMAIN(("dEvian: Free devian remove"));
  dEvianM->devians = evas_list_remove (dEvianM->devians,
					   devian);
  DMAIN(("Rename dEvians"));
  devian_devian_set_id(NULL, 0, NULL);
  DMAIN(("dEvian: Free devian free"));

  if (devian->id)
    E_FREE(devian->id);
  E_FREE(devian);

  e_config_save_queue();

  DMAIN(("dEvian: Free devian OK"));
}

/**
 * Delete all dEvians, with force option (now)
 */
void devian_devian_del_all(void)
{
  while (dEvianM->devians)
    {
      DMAIN(("dEvian: devian_del"));
      devian_devian_del(evas_list_data(dEvianM->devians), 1);
    }
}

/**
 * Set the name of one / all devian(s)
 *
 * @param devian If isnt NULL, set the name only for the pointed devian
 * The name is choosen with -source, and you can add -extra
 * @param source Type of the source of the devian indicated as -devian
 * @param extra You can add an extra to one / all devian(s) name's, or set it to null
*/
void devian_devian_set_id(dEvian *devian, int source, char *extra)
{
  char *id;
  int n;

  if (!devian)
    {
      Evas_List *l;
      dEvian *d;

      n=1;
      for (l=dEvianM->devians; l; l=evas_list_next(l))
	{
	  id = E_NEW(char, DEVIAN_ID_LEN);
	  d = evas_list_data(l);
	  if (d->id && d->conf)
	    {
	      E_FREE(d->id);
	      switch (d->conf->source_type)
		{
		case SOURCE_PICTURE:
		  snprintf(id, DEVIAN_ID_LEN, "Picture %d", n);
		  break;
		case SOURCE_RSS:
		  snprintf(id, DEVIAN_ID_LEN, "Rss %d", n);
		  break;
		case SOURCE_NO:
		  snprintf(id, DEVIAN_ID_LEN, "No source %d", n);
		  break;
		}
	    }
	  else
	    {
	      if (d->id)
		E_FREE(d->id);
	      snprintf(id, DEVIAN_ID_LEN, "Invalid devian %d", n);
	    }
	  d->id = id;
	  devian_container_update_id_devian(d);
	  n++;
	}
    }
  else
    {
      id = E_NEW(char, DEVIAN_ID_LEN);
      n = evas_list_count(dEvianM->devians);
      switch (source)
	{
	case SOURCE_PICTURE:
	  if (extra)
	    snprintf(id, DEVIAN_ID_LEN, "Picture %d %s", n, extra);
	  else
	  snprintf(id, DEVIAN_ID_LEN, "Picture %d", n);
	  break;
	case SOURCE_RSS:
	  if (extra)
	    snprintf(id, DEVIAN_ID_LEN, "Rss %d %s", n, extra);
	  else
	  snprintf(id, DEVIAN_ID_LEN, "Rss %d", n);
	  break;
	case SOURCE_NO:
	  if (extra)
	    snprintf(id, DEVIAN_ID_LEN, "No source %d %s", n, extra);
	  else
	    snprintf(id, DEVIAN_ID_LEN, "No source %d", n);
	  break;
	default: 
	  if (extra)
	    snprintf(id, DEVIAN_ID_LEN, "Invalid dEvian %d %s", n, extra);
	  else
	    snprintf(id, DEVIAN_ID_LEN, "Invalid dEvian %d", n);
	  break;
	}
      if (devian->id)
	E_FREE(devian->id);
      devian->id = id;
      if (devian->container)
	devian_container_update_id_devian(devian);
    }

  return;
}

/**
 * Initialise the e_lib
 * 
 * @return Name of the current X display
 */
char *devian_display_init(void)
{
  char *display = NULL;
  char *tmp;

  tmp = getenv("DISPLAY");
  if (tmp) display = strdup(tmp);

  /* make sure the display var is of the form name:0.0 or :0.0 */
  if (display)
    {
      char *p;
      char buf[1024];

      p = strrchr(display, ':');
      if (!p)
	{
	  snprintf(buf, sizeof(buf), "%s:0.0", display);
	  free(display);
	  display = strdup(buf);
	}
      else
	{
	  p = strrchr(p, '.');
	  if (!p)
	    {
	      snprintf(buf, sizeof(buf), "%s.0", display);
	      free(display);
	      display = strdup(buf);
	    }
	}
    }
  else
    display = strdup(":0.0");

  /* Init E Lib */
  if (display)
    e_lib_init(display);
  else
    {
      display = strdup(":0.0");
      e_lib_init(display);
    }

  return display;
}

/**
 * Shutdown e_lib
 */
void devian_display_shutdown(void)
{
  free(dEvianM->display);
  dEvianM->display = NULL;
  e_lib_shutdown();
}

/**
 * Loads a specific part of the current theme to an edje object
 * 
 * @param edje_obj The edje object
 * @param part Name of the part to load in current theme
 * @param type Type of the theme to use (picture, rss, popup ...)
 * @return The name of the theme used when loading the part, or NULL if fails
 */

const char *devian_devian_edje_load(Evas_Object *edje_obj, char *part, int type)
{
  char path[DEVIAN_MAX_PATH];
  const char *theme;

  switch (type)
    {
    case DEVIAN_THEME_TYPE_PICTURE:
      theme = dEvianM->conf->theme_picture;
      break;
    case DEVIAN_THEME_TYPE_RSS:
      theme = dEvianM->conf->theme_rss;
      break;
    case DEVIAN_THEME_TYPE_POPUP:
      theme = dEvianM->conf->theme_popup;
      break;
    default:
      theme = dEvianM->conf->theme_popup;
    }

  snprintf(path, DEVIAN_MAX_PATH, "%s/%s/%s%s",
	   e_module_dir_get(dEvianM->module), "themes",
	   theme, ".edj");
      if ( !edje_object_file_set(edje_obj, path, part) )
    {
      char buf[4096];
      snprintf(buf, sizeof(buf),
               _(
                 "<hilight>Error !</hilight><br>"
                 "Did not find <hilight>theme</hilight> %s group %s"),
               path, part);
      e_module_dialog_show(_("dEvian Module Error"), buf);
      return NULL;
    }
  DMAIN(("Edje loaded %s group %s\n" , path, part));
  
  return theme;
}

/**
 * Makes a charset convertion to UTF8, and creates an evas_stringshare with the result
 * 
 * @param text The text to use
 * @param tfree Free the text after ?
 * @return The evas_strinshare in utf8 if translation was ok, evas_stringshare of original text otherwise
 */
const char *devian_ss_utf8_add(char *text, int tfree)
{
  char *utf8;
  const char *ss;

  utf8 = ecore_txt_convert("iso-8859-1", "UTF-8", text);
  if (!utf8)
    {
      ss = evas_stringshare_add(text);
    }
  else
    {
      ss = evas_stringshare_add(utf8);
      E_FREE(utf8);
    }
  if (tfree)
    E_FREE(text);

  return ss;
}
