#include "dEvian.h"

static void _devian_source_del(dEvian *devian);

/* Public functions */

/**
 * Change de source of a dEvian
 *
 * @param devian The dEvian
 * @return 0 on success, 1 on fail
 */
int devian_source_change(dEvian *devian, int source)
{
  if (devian->container_func.is_in_transition(devian))
    {
      DMAIN(("--- Wanna source change, but already in transition ---"));
      return 1;
    }

  /* CHECK: if already one source, remove it, except if its the one we want */
  if (devian->source)
    {
      if (devian->conf->source_type == source)
	return 1;

      _devian_source_del(devian);
      
      if (devian->dialog_conf)
	devian_config_dialog_devian_shutdown(devian);
    }

  /* Add the new container to the devian */
  switch (source)
    {
    case SOURCE_PICTURE:
      {
	if (!dEvianM->source_picture_count)
	  {
	    /* Init pictures subsystem */
	    if ( devian_data_picture_list_local_init() )
	      {
		fprintf(stderr, "dEvian: can't init picture list (local) !\n");
		return 1;
	      }
	    if ( !devian_data_picture_cache_init() )
	      {
		fprintf(stderr, "dEvian: can't init picture cache !\n");
		return 1;
	      }
	  }
	/* Add the picture */
	if ( devian_source_picture_add(devian) )
	  return 1;
	/* Chech good theme */
	devian_container_theme_check(devian, &dEvianM->conf->theme_picture);
	break;
      }

    case SOURCE_RSS:
      {
	/* Add the rss */
	if ( !devian_source_rss_add(devian) )
	  return 1;
	/* Chech good theme */
	devian_container_theme_check(devian, &dEvianM->conf->theme_rss);
	break;
      }

    case SOURCE_TEXT:
      {
	return 1;
      }

    case SOURCE_NO:
      {
	/* I want no source attached, thank you */
	return 0;
      }

    default : return 1;
    }
  
  return 0;
}

void devian_source_detach(dEvian *devian, int part)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      devian_data_picture_cache_detach(devian->source, part);
      break;
    case SOURCE_RSS:
      devian_data_rss_prev_detach(devian->source, part);
      break;
    case SOURCE_TEXT:
      break;
    }
  return;
}

int devian_source_viewer(dEvian *devian)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_viewer(devian->source);
    case SOURCE_RSS:
      return devian_source_rss_viewer(devian->source);
    case SOURCE_TEXT:
      return 0;
    }

  return 0;
}

/*
  Get the evas objects of a source
  Return:
    - 1 on success
    - 0 on fail
*/
int devian_source_evas_object_get(dEvian *devian,
				  Evas_Object **obj0,
				  Evas_Object **obj1)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_evas_object_get(devian, obj0, obj1);
    case SOURCE_RSS:
      return devian_source_rss_evas_object_get(devian, obj0, obj1);
    case SOURCE_TEXT:
	break;
    case SOURCE_NO:
	break;
    }

  return 0;
}

int devian_source_original_size_get(dEvian *devian,
				   int part, int *w, int *h)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_original_size_get(devian->source,
						    part, w, h);
    case SOURCE_RSS:
      return devian_source_rss_original_size_get(devian->source,
						 part, w, h);
    case SOURCE_TEXT:
	break;
    case SOURCE_NO:
	break;
    }

  return 0;
}

void devian_source_dialog_infos(dEvian *devian)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      devian_source_picture_dialog_infos(devian->source);
      break;
    case SOURCE_RSS:
      devian_source_rss_dialog_infos(devian->source);
      break;
    case SOURCE_TEXT:
      break;
    }
  
  return;
}

char *devian_source_name_get(dEvian *devian, int part)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_name_get(devian->source, part);
    case SOURCE_RSS:
      return devian_source_rss_name_get(devian->source);
    case SOURCE_TEXT:
      break;
    }
  return NULL;
}

char *devian_source_infos_get(dEvian *devian, int part)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_infos_get(devian->source, part);
    case SOURCE_RSS:
      return devian_source_rss_infos_get(devian->source);
    case SOURCE_TEXT:
      break;
    }

  return NULL;
}

void devian_source_idle_timer_change(dEvian *devian, int active, int time)
{
  return;
}
int devian_source_idle_refresh(dEvian *devian)
{
  return 0;
}
int devian_source_idle_set_bg(dEvian *devian)
{
  return 0;
}
int devian_source_idle_viewer(dEvian *devian)
{
  return 0;
}
int devian_source_idle_gui_update(dEvian *devian)
{
  return 0;
}

/* Private functions */

static void _devian_source_del(dEvian *devian)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_NO: break;
    case SOURCE_PICTURE:
      devian_source_picture_del(devian->source);
      if (!dEvianM->source_picture_count)
	{
	  /* Shutdown pictures subsystem */
	  devian_data_picture_cache_shutdown();
	  devian_data_picture_list_local_shutdown();
	}
      break;
    case SOURCE_RSS:
      devian_source_rss_del(devian->source);
      break;
    case SOURCE_TEXT: break;
    }

  /* Actions */
  devian->source_func.timer_change = devian_source_idle_timer_change;
  devian->source_func.refresh = devian_source_idle_refresh;
  devian->source_func.set_bg = devian_source_idle_set_bg;
  devian->source_func.viewer = devian_source_idle_viewer;
  devian->source_func.gui_update = devian_source_idle_gui_update;
}
